import os

import click
from click.testing import CliRunner
from ksfctl.generate.cpp_generator import cpp_gen


@click.group(context_settings={'help_option_names': ['-h', '--help']})
@click.pass_context
def cli(ctx):
    ctx.help_option_names += ['-h']
    pass


@cli.group(name='parse', help="解析ksf文件到其他语言，目前支持(cpp, python, java, node.js, go)")
@click.pass_context
def parse(ctx):
    ctx.help_option_names += ['-h']
    pass


@parse.command(name='cpp')
@click.pass_context
@click.argument('ksf_files', nargs=-1, required=True, type=str)
@click.option('-i', '--include', '--include-path', multiple=True, help='ksf协议文件搜索路径')
@click.option('-d', '--dir', '--dest', 'destination_dir', multiple=False, help='生成的头文件存放的路径')
@click.option('--replace_ns', '--replace-ns', multiple=False, help='(将被废弃)替换namespace')
@click.option('--replace-namespace', nargs=2, multiple=True, type=str, help='(推荐)将指定命名空间替换为另一个命名空间')
@click.option('--replace-include-dir', nargs=2, multiple=True, type=str, help='(推荐)替换头文件路径')
@click.option('--check-default/--no-check-default', default=True, help='是否打包默认值')
@click.option('--ksf/--no-ksf', 'ksf', is_flag=True, default=False, help='是否ksf内部模块')
@click.option('--json/--no-json', 'json', is_flag=True, default=True, help='是否生成Json格式')
@click.option('--sql/--no-sql', 'sql', is_flag=True, default=False, help='是否生成Sql接口')
@click.option('--rpc/--no-rpc', 'rpc', is_flag=True, default=True, help='是否生成RPC接口')
@click.option('--current-priority/--no-current-priority', 'current_priority', is_flag=True, default=True,
              help='是否优先使用当前目录')
@click.option('--trace/--no-trace', 'trace', is_flag=True, default=False, help='是否需要调用链追踪逻辑')
@click.option('--push/--no-push', 'push', is_flag=True, default=True, help='是否需要推送接口')
@click.option('--param-rvalue-ref/--no-param-rvalue-ref', 'param_rvalue_ref', is_flag=True, default=False,
              help='是否参数使用右值引用')
@click.option('--with-ksf', 'ksf', is_flag=True, flag_value=True, default=False, help='ksf内部模块')
@click.option('--unjson', 'json', is_flag=True, flag_value=False, default=True, help='不生成Json格式')
@click.option('--os', 'rpc', is_flag=True, default=True, flag_value=False, help='不生成RPC接口')
@click.option('--currentPriority', 'current_priority', is_flag=True, flag_value=True, default=True,
              help='优先使用当前目录')
@click.option('--without-trace', 'trace', is_flag=True, flag_value=False, default=False, help='不需要调用链追踪')
@click.option('--with-push', 'push', is_flag=True, flag_value=True, default=True, help='需要推送接口')
@click.option('--with-param-rvalue-ref', 'param_rvalue_ref', is_flag=True, flag_value=True, default=False,
              help='参数使用右值引用')
def parse_cpp(ctx, ksf_files, include, destination_dir, replace_ns, replace_namespace, replace_include_dir, **kwargs):
    ctx.help_option_names += ['-h']
    """命令的描述"""
    click.echo(f'文件列表: {ksf_files}')

    """将指定命名空间替换为另一个命名空间"""
    if replace_namespace:
        for ns in replace_namespace:
            origin, actual = ns
            click.echo(f'将命名空间 {origin} 替换为 {actual}。')
    elif replace_ns:
        replace_namespace = []
        for replace_namespace_str in replace_ns.split(";"):
            origin, actual = replace_namespace_str.split("/")
            replace_namespace.append((origin, actual))
            click.echo(f'将命名空间 {origin} 替换为 {actual}。')
    else:
        click.echo('未指定任何命名空间的置换。')

    """替换头文件路径"""
    if replace_include_dir:
        for ns in replace_namespace:
            origin, actual = ns
            click.echo(f'将头文件路径 {origin} 替换为 {actual}。')
    else:
        click.echo('未指定任何头文件路径的置换。')

    """生成文件位置"""
    if destination_dir:
        click.echo(f'生成文件位置: {destination_dir}')
    else:
        destination_dir = os.getcwd()
        click.echo(f'生成文件位置: {destination_dir}')

    """头文件包含路径"""
    if include:
        click.echo(f'包含路径: {include}')
    else:
        click.echo(f'未指定包含路径，将只在当前路径搜索')

    """是否检测默认值"""
    click.echo(f"是否检测默认值：{kwargs['check_default']}")

    """是否ksf内部模块"""
    click.echo(f"是否ksf内部模块：{kwargs['ksf']}")

    """是否生成Json序列化接口"""
    click.echo(f"是否生成Json序列化接口：{kwargs['json']}")

    """是否生成Sql接口"""
    click.echo(f"是否生成Sql接口：{kwargs['sql']}")

    """是否生成RPC接口"""
    click.echo(f"是否生成RPC接口：{kwargs['rpc']}")

    """是否优先使用当前目录"""
    click.echo(f"是否优先使用当前目录：{kwargs['current_priority']}")

    """是否需要调用链追踪逻辑"""
    click.echo(f"是否需要调用链追踪逻辑：{kwargs['trace']}")

    """是否参数使用右值引用"""
    click.echo(f"是否参数使用右值引用：{kwargs['param_rvalue_ref']}")

    """解析所有的flags，携带with_头"""
    kwargs_with_prefix = {}
    for k, v in kwargs.items():
        kwargs_with_prefix.update({f'with_{k}': v})

    cpp_gen(files=ksf_files, include_dirs=include, destination_dir=destination_dir, flags=kwargs_with_prefix)


if __name__ == '__main__':
    cli()
    exit(0)

    runner = CliRunner()
    result1 = runner.invoke(cli, ['-h'])
    assert result1.exit_code == 0

    result2 = runner.invoke(cli, ['--help'])
    assert result2.exit_code == 0





import ply.lex as lex


# 定义 KSF IDL 文件中的词法符号
class KsfLexer:
    states = (
        ('comment', 'exclusive'),
    )

    tokens = (
        'INCLUDE',
        'INTERFACE',
        'MODULE',
        'STRUCT',
        'ENUM',
        'CONST',
        'KEY',

        # 符号
        'LEFT_PAREN',
        'RIGHT_PAREN',
        'LEFT_BRACE',
        'RIGHT_BRACE',
        "LEFT_SQUARE",
        "RIGHT_SQUARE",
        "LEFT_ANGLE_BRACKET",
        "RIGHT_ANGLE_BRACKET",
        'SEMICOLON',
        'COMMA',
        # 'COLON',
        'EQUALS',
        # 'DOT',
        # 输出参数前缀
        'OUTPUT',

        'IDENTIFIER',
        'STRING_CONSTANT',
        'INT_CONSTANT',
        'RADIX_CONSTANT',
        'EXPR_CONSTANT',

        # 限定
        'UNSIGNED',

        # 字段模式
        "REQUIRED",
        "OPTIONAL",

        # 基本类型
        'INT',
        'LONG',
        'FLOAT',
        'DOUBLE',
        'SHORT',
        'BOOL',
        'CHAR',
        'STRING',
        'VECTOR',
        'MAP',

        'COMMENT_MULTILINE',
        'COMMENT_LINE',
    )

    # 定义词法规则优先级
    precedence = (
        ('left', 'EXPR_CONSTANT'),
    )

    # 定义 KSF IDL 文件中的词法规则
    t_LEFT_PAREN = r'\('
    t_RIGHT_PAREN = r'\)'
    t_LEFT_BRACE = r'\{'
    t_RIGHT_BRACE = r'\}'
    t_LEFT_SQUARE = r'\['
    t_RIGHT_SQUARE = r'\]'
    t_LEFT_ANGLE_BRACKET = r'\<'
    t_RIGHT_ANGLE_BRACKET = r'\>'
    t_SEMICOLON = r';'
    t_COMMA = r','
    # t_COLON = r':'
    t_EQUALS = r'='

    # t_DOT = r'\.'
    t_comment_ignore = ''

    # 定义多行注释的规则，进入 comment 状态
    def __init__(self):
        self.lexer = None

    def t_comment(self, t):
        r'\/\*'
        t.lexer.begin('comment')

    def t_comment_COMMENT_MULTILINE(self, t):
        r"[\s\S]*?\*/"
        t.value = t.value[:-2]

        t.lexer.begin('INITIAL')
        return t

    def t_comment_error(self, t):
        t.lineno, t.lexpos = self._find_position(t.lexer.lexdata, t.lexpos)
        print("Illegal character '%s' in MULTI_LINE_COMMENT" % t.value[0])
        t.lexer.skip(1)

    def t_IDENTIFIER(self, t):
        r'[a-zA-Z_][a-zA-Z0-9_]*(::[a-zA-Z_][a-zA-Z0-9_]*)?'
        t.type = {
            "bool": "BOOL",
            "require": "REQUIRED",
            "optional": "OPTIONAL",
            "enum": "ENUM",
            "struct": "STRUCT",
            "interface": "INTERFACE",
            "module": "MODULE",
            "map": "MAP",
            "vector": "VECTOR",
            "string": "STRING",
            "byte": "CHAR",
            "short": "SHORT",
            "long": "LONG",
            "double": "DOUBLE",
            "float": "FLOAT",
            "int": "INT",
            "unsigned": "UNSIGNED",
            "key": "KEY",
            "out": "OUTPUT",
            "const": "CONST",

        }.get(t.value, "IDENTIFIER")
        return t

    # 定义字符串
    def t_STRING_CONSTANT(self, t):
        r""""[^"]*\""""
        t.value = t.value[1:-1]
        return t

    # 定义多进制的正则表达式
    def t_RADIX_CONSTANT(self, t):
        r'(-?(0[bB][01]+|0[oO][0-7]+|0[xX][0-9a-fA-F]+|\d+(\.\d+)?([eE][-+]?\d+)?))'
        return t

    # 定义科学计数的正则表达式
    def t_EXPR_CONSTANT(self, t):
        r'(-?\d+(\.\d+)?([eE][-+]?\d+)?)'
        return t


    # 定义整数的正则表达式
    def t_INT_CONSTANT(self, t):
        r'\d+'
        return t

    def t_COMMENT_LINE(self, t):
        r"""//.*"""
        t.value = t.value[2:]
        return t

    # 定义包含关键字的规则
    def t_INCLUDE(self, t):
        r'\#include'
        return t

    # 定义忽略的字符，包括空格、制表符和换行符·
    t_ignore = ' \t\r\n'

    def t_newline(self, t):
        r"\n+"
        t.lexer.lineno += len(t.value)

    # 定义错误处理函数，打印错误信息
    def t_error(self, t):
        t.lineno, t.lexpos = self.find_position(t.lexer.lexdata, t.lexpos)
        print("Illegal character '%s'" % t.value[0])
        t.lexer.skip(1)

    # 编译lexer
    def build(self, **kwargs):
        self.lexer = lex.lex(module=self, **kwargs)

    def test(self, data):
        self.lexer.input(data)
        while True:
            tok = self.lexer.token()
            if not tok:
                break
            tok.lineno, tok.lexpos = self.find_position(data, tok)
            print(tok)

    @staticmethod
    def find_position(text, token):
        """
        获取当前token所在位置
        :param text:
        :param token:
        :return:
        """
        index = token.lexpos
        lines = text.splitlines(keepends=True)
        line_number = 0
        for line in lines:
            line_length = len(line)
            if index < line_length:
                column_number = index
                break
            index -= line_length
            index -= 1  # 减去回车符
            line_number += 1
        else:
            return -1, -1
        return line_number + 1, column_number + 2


if __name__ == '__main__':
    from pathlib import Path
    lexer = KsfLexer()
    lexer.build()
    # 定义 KSF IDL 文件
    ksf_idl = '''
#include "123.ksf"
module MyModule {
    /*my interface*/
    interface MyInterface {
        int add(int a, int b);
        void print(string message);
    }
}
    '''

    file = Path('../../example/const_definition.ksf')
    with file.open() as f:
        # 将 KSF IDL 文件传递给词法分析器
        lexer.test(f.read())



from collections import defaultdict
from pathlib import Path


class KsfFile:
    def __init__(self, file):
        self.file = file
        self.elements = []
        self.includes = []

    def add_enum(self, enum):
        self.elements.append(enum)

    def add_interface(self, interface):
        self.elements.append(interface)

    def add_struct(self, struct):
        self.elements.append(struct)

    def add_const(self, const):
        self.elements.append(const)

    def add_include(self, inc):
        self.includes.append(inc)


class KsfEnumMember:
    def __init__(self, file, module, enum, name, value, comment=''):
        self.file = file
        self.module = module
        self.name = name
        self.enum = enum
        self.value = value
        self.comment = comment

        self.id = '.'.join([self.module, self.enum, self.name])


class KsfEnum:
    def __init__(self, file, module, name, member, comment=None):
        self.file = file
        self.module = module
        self.name = name
        self.comment = comment
        self.member = []
        for m in member:
            ksf_enum_member = KsfEnumMember(file, module, name, m['name'], m['value'], m['comment'])
            self.member.append(ksf_enum_member)

        self.id = '.'.join([self.module, self.name])


class KsfField:
    def __init__(self, file, module, struct, tag, is_required, value_type, name, default, comment=None):
        self.file = file
        self.module = module
        self.struct = struct
        self.tag = tag
        self.is_required = is_required
        self.value_type = value_type
        self.name = name
        self.default = default
        self.comment = comment

        self.id = '.'.join([self.module, self.struct, self.name])


class KsfStruct:
    def __init__(self, file, module, name, variable, comment=None):
        self.key_fields = []
        self.file = file
        self.module = module
        self.name = name
        self.comment = comment
        self.variable = {}
        for v in variable:
            ksf_field = KsfField(file, module, name, v['tag'], v['is_required'], v['value_type'], v['name'],
                                 v['default'] if 'default' in v else None, v['comment'])
            self.variable[ksf_field.id] = ksf_field

        self.id = '.'.join([self.module, self.name])

    def add_key(self, arg: list):
        self.key_fields = arg


class KsfVariable:
    def __init__(self, file, module, interface, operator, value_type, name, index):
        self.file = file
        self.module = module
        self.interface = interface
        self.operator = operator
        self.value_type = value_type
        self.name = name
        self.index = index

        self.id = '.'.join([self.module, self.interface, self.operator, self.name])


class KsfOperator:
    def __init__(self, file, module, interface, name, return_type, variable, comment=None):
        self.file = file
        self.module = module
        self.interface = interface
        self.name = name
        self.return_type = return_type
        self.comment = comment

        self.input = {}
        self.output = {}
        self.ordered_var = []

        index = 1
        for v in variable:
            ksf_variable = KsfVariable(file, module, interface, name, v['value_type'], v['name'], index)
            if v['is_output']:
                self.ordered_var.append((ksf_variable.name, True))
                self.output[ksf_variable.name] = ksf_variable
            else:
                self.ordered_var.append((ksf_variable.name, False))
                self.input[ksf_variable.name] = ksf_variable

            index += 1

        self.id = '.'.join([self.module, self.interface, self.name])


class KsfInterface:
    def __init__(self, file, module, name, operator, comment=None):
        self.file = file
        self.module = module
        self.name = name
        self.comment = comment
        self.operator = {}

        for o in operator:
            ksf_operator = KsfOperator(file, module, name, o['name'], o['return_type'], o['variable'], o['comment'])
            self.operator[ksf_operator.name] = ksf_operator

        self.id = '.'.join([self.module, self.name])


class KsfConst:
    def __init__(self, file, module, name, value_type, value, comment=''):
        self.file = file
        self.module = module
        self.name = name
        self.value_type = value_type
        self.value = value
        self.comment = comment

        self.id = '.'.join([self.module, self.name])


class KsfBuildInType:
    def __init__(self, name):
        self.id = 'BuildInType.' + name
        self.name = name


class KsfMapType:
    def __init__(self, name, key_type, value_type):
        self.key_type = key_type
        self.value_type = value_type
        super().__init__('MapType.' + name, name)


class KsfVectorType:
    def __init__(self, name, element_type):
        self.element_type = element_type
        super().__init__('VectorType.' + name, name)


class KsfStructType:
    def __init__(self, identity, name, fields, comment='', *args, **kwargs):
        super().__init__(identity, name, *args, **kwargs)
        self.fields = fields
        self.comment = comment


class KsfArrayType:
    pass


def singleton(cls):
    instances = {}

    def get_instance(*args, **kwargs):
        if cls not in instances:
            instances[cls] = cls(*args, **kwargs)
        return instances[cls]

    return get_instance


class Ksf:
    natives = {
        "string": KsfBuildInType(name='string'),
        "int": KsfBuildInType(name='int'),
        "uint": KsfBuildInType(name='uint'),
        "bool": KsfBuildInType(name='bool'),
        "short": KsfBuildInType(name='int'),
        "ushort": KsfBuildInType(name='ushort'),
        "long": KsfBuildInType(name='long'),
        "float": KsfBuildInType(name='float'),
        "double": KsfBuildInType(name='double'),
        "byte": KsfBuildInType(name='byte'),
    }

    files = {}

    file2module = defaultdict(list)
    module2file = defaultdict(list)

    enums = {}
    file2enum = defaultdict(list)  # 文件到枚举的映射
    module2enum = defaultdict(list)  # 模块到枚举的映射

    structs = {}
    file2struct = defaultdict(list)  # 文件到结构体的映射
    module2struct = defaultdict(list)  # 模块到结构体的映射

    interfaces = {}
    file2interface = defaultdict(list)  # 文件到接口的映射
    module2interface = defaultdict(list)  # 模块到接口的映射

    consts = {}
    file2const = defaultdict(list)  # 文件到常量的映射
    module2const = defaultdict(list)  # 模块到常量的映射

    all_elements = []

    @classmethod
    def add_enum(cls, ksf_enum: KsfEnum):
        cls.enums[ksf_enum.id] = ksf_enum
        cls.file2enum[ksf_enum.file].append(ksf_enum.id)
        cls.module2enum[ksf_enum.module].append(ksf_enum.id)
        cls.files[ksf_enum.file].add_enum(ksf_enum)
        cls.all_elements.append(ksf_enum)

    @classmethod
    def add_struct(cls, ksf_struct: KsfStruct):
        cls.structs[ksf_struct.id] = ksf_struct
        cls.file2struct[ksf_struct.file].append(ksf_struct.id)
        cls.module2struct[ksf_struct.module].append(ksf_struct.id)
        cls.files[ksf_struct.file].add_struct(ksf_struct)
        cls.all_elements.append(ksf_struct)

    @classmethod
    def add_interface(cls, ksf_interface: KsfInterface):
        cls.interfaces[ksf_interface.id] = ksf_interface
        cls.file2interface[ksf_interface.file].append(ksf_interface.id)
        cls.module2interface[ksf_interface.module].append(ksf_interface.id)
        cls.files[ksf_interface.file].add_interface(ksf_interface)
        cls.all_elements.append(ksf_interface)

    @classmethod
    def add_struct_key(cls, module, struct, arg):
        cls.structs[module + '.' + struct].add_key(arg)

    @classmethod
    def add_constant(cls, ksf_const: KsfConst):
        cls.consts[ksf_const.id] = ksf_const
        cls.file2interface[ksf_const.file].append(ksf_const.id)
        cls.module2interface[ksf_const.module].append(ksf_const.id)
        cls.files[ksf_const.file].add_const(ksf_const)
        cls.all_elements.append(ksf_const)

    @classmethod
    def add_file(cls, file: Path):
        ksf_file = KsfFile(file)
        cls.files[file.name] = ksf_file

    @classmethod
    def add_include(cls, file: Path, include: Path):
        cls.files[file.name].add_include(include)


from collections import defaultdict, deque


class Graph:
    def __init__(self):
        self.graph = defaultdict(list)
        self.visited = {}
        self.path = []

    def add_edge(self, u, v):
        if u not in self.graph:
            self.graph[u] = []
        if v not in self.graph:
            self.graph[v] = []

        if v not in self.graph[u]:
            self.graph[u].append(v)
            return True
        else:
            return False

    def find_cycle(self):
        # 初始化所有节点的访问状态
        for node in self.graph:
            self.visited[node] = False

        # 从每个节点开始搜索，直到找到第一个环
        for node in self.graph:
            if node in self.visited and not self.visited[node]:
                self.path = []
                if self._dfs(node):
                    return self.path[::-1]  # 反转环的顺序，以使其符合正常的阅读顺序

        return None

    def _dfs(self, node):
        self.visited[node] = True
        self.path.append(node)

        # 沿着出度的路径进行搜索
        for neighbor in self.graph[node]:
            if neighbor in self.visited and not self.visited[neighbor]:
                # 如果邻接节点还没有被访问，则继续搜索
                if self._dfs(neighbor):
                    return True

            elif neighbor in self.path:
                # 如果当前节点的邻接节点已经在路径中，则说明存在环
                index = self.path.index(neighbor)
                self.path = self.path[index:]
                return True

        self.path.pop()
        return False

    def bfs(self, start, callback):
        # 标记所有节点的访问状态
        visited = {node: False for node in self.graph}

        # 创建队列，并将起点加入队列中
        queue = deque()
        queue.append(start)

        # 广度优先搜索算法
        node = None
        while queue:
            node = queue.popleft()

            # 如果当前节点还没有被访问，则将其标记为已访问，并输出
            if node in visited and not visited[node]:
                visited[node] = True
                callback(node)

                # 将所有邻接节点加入队列中
                for neighbor in self.graph.get(node, []):
                    queue.append(neighbor)

        return visited

    def get_all_prev(self, curr, with_self=True):
        """
        获取所有前向节点
        :return:
        """
        nodes = set()
        self.bfs(curr, lambda a: nodes.add(a) if with_self or a != curr else None)

        return nodes

    def bfs_dag(self, callback):
        # perform topological sorting to get linear ordering of vertices
        in_degree = {u: 0 for u in self.graph}
        for u in self.graph:
            for v in self.graph[u]:
                in_degree[v] += 1
        queue = deque([u for u in self.graph if in_degree[u] == 0])
        visited = []
        while queue:
            u = queue.popleft()
            visited.append(u)
            for v in self.graph[u]:
                in_degree[v] -= 1
                if in_degree[v] == 0:
                    queue.append(v)
        # perform BFS traversal in topological order
        visited_nodes = set()
        bfs_queue = deque(visited)
        visited_nodes.update(visited)
        while bfs_queue:
            current_node = bfs_queue.popleft()
            callback(current_node)
            for neighbor in self.graph[current_node]:
                if neighbor not in visited_nodes:
                    bfs_queue.append(neighbor)
                    visited_nodes.add(neighbor)

    def has_node(self, node):
        return node in self.graph



from hashlib import md5
from ksfctl.parser.ksf_parser import *
from ksfctl.parser.ksf_yacc import generate


class CppGenerator:
    def __init__(self, ast, filename, dest, **kwargs):
        self.__dict__.update(kwargs)
        self.ast = ast
        file = Path(filename)
        real_name = file.name
        self.ast_in_file = ast.files[real_name]

        Path(dest).mkdir(parents=True, exist_ok=True)
        self.generated_file = (Path(dest) / (real_name[:-4] + '.h'))

        self.inc_ordered = [
            "<string>",
            "<vector>",
            "<set>",
            "<unordered_set>",
            "<map>",
            "<unordered_map>",
            "<kup/Ksf.h>",
            "<kup/KsfJson.h>",
            "<servant/Agent.h>",
            "<servant/Servant.h>",
            "<promise/promise.h>",
        ]
        self.inc_native = {
            "<string>": True,
            "<vector>": False,
            "<set>": False,
            "<unordered_set>": False,
            "<map>": False,
            "<unordered_map>": False,
            "<kup/Ksf.h>": False,
            "<kup/KsfJson.h>": False,
            "<servant/Agent.h>": False,
            "<servant/Servant.h>": False,
            "<promise/promise.h>": False,
        }
        self.inc_depends = []
        self.file_str = ''
        self.curr_tab = ''

    def __getattr__(self, name):
        if name.startswith('with_') and not hasattr(self, 'name'):
            return False
        return super().__getattr__(name)

    def add_include(self, header):
        if header in self.inc_native:
            self.inc_native[header] = True
        else:
            self.inc_depends.append(header)

    def parse_header(self):
        output = ""
        for header in self.inc_ordered:
            if self.inc_native[header]:
                output += "#include " + header + "\n"

        output += "\n"

        for header in self.inc_depends:
            output += "#include " + header + "\n"
        return output

    def inc_tab(self):
        self.curr_tab += '\t'

    def del_tab(self):
        self.curr_tab = self.curr_tab[:-1]

    @staticmethod
    def get_type_id(curr_module, value_type):
        return f"{curr_module if value_type['module'] is None else value_type['module']}.{value_type['name']}"

    def is_movable_type(self, module, value_type):
        """判断是否是可以进行std::move的类型"""
        if value_type['type'] == 'native':
            return value_type['name'] == 'string'
        elif value_type['type'] == 'class':

            if hasattr(value_type, 'module'):
                class_full_name = value_type['module'] + "." + value_type['name']
            else:
                class_full_name = module + "." +value_type['name']

            return class_full_name not in self.ast.enums
        elif value_type['type'] == 'vector':
            return True
        elif value_type['type'] == 'map':
            return True
        else:
            return False

    def parse_type(self, module, value_type):
        if value_type['type'] == 'native':
            if value_type['name'] == 'string':
                self.add_include(f"<string>")
                return 'std::string'
            elif value_type['name'] == 'int':
                return f'{"u" if value_type["unsigned"] else ""}int{value_type["bit"]}_t'
            elif value_type['name'] == 'float':
                return value_type['name']
            elif value_type['name'] == 'double':
                return value_type['name']
            elif value_type['name'] == 'bool':
                return value_type['name']
            else:
                raise SyntaxError(f"不支持解析的字段类型[{value_type}]")
        elif value_type['type'] == 'class':
            if hasattr(value_type, 'module') and module != value_type['module']:
                return value_type['module'] + "::" + value_type['name']
            else:
                return value_type['name']
        elif value_type['type'] == 'vector':
            if value_type['is_ordered'] and not value_type['is_hashed'] and not value_type['is_unique_member']:
                true_type = 'std::vector'
                self.add_include('<vector>')
            elif value_type['is_ordered'] and not value_type['is_hashed'] and value_type['is_unique_member']:
                true_type = 'std::set'
                self.add_include('<set>')
            elif not value_type['is_ordered'] and value_type['is_hashed'] and value_type['is_unique_member']:
                true_type = 'std::unordered_set'
                self.add_include('<unordered_set>')
            else:
                raise SyntaxError(f"不支持解析的字段类型[{value_type}]")

            return f"{true_type}<{self.parse_type(module, value_type['value_type'])}>"
        elif value_type['type'] == 'map':
            if not value_type['is_ordered'] and value_type['is_hashed'] and value_type['is_unique_member']:
                true_type = 'std::unordered_map'
                self.add_include('<unordered_map>')
            elif value_type['is_ordered'] and not value_type['is_hashed'] and value_type['is_unique_member']:
                true_type = 'std::map'
                self.add_include('<map>')
            else:
                raise SyntaxError(f"不支持解析的字段类型[{value_type}]")
            return f"{true_type}<{self.parse_type(module, value_type['key_type'])}, {self.parse_type(module, value_type['value_type'])}>"

    def parse_comment_above(self, comment, tab=None):
        if comment is None or comment == '':
            return ''

        if tab is None:
            tab = self.curr_tab

        comment_lines = comment.split('\n')
        if len(comment_lines) == 1:
            return f"{tab}//{comment}\n"
        else:
            import re
            matches = re.findall(r"\s+\*\s+(.*)\n", f"/*{comment}*/")
            comment_str = f'{tab}/**\n'

            # 将每行注释内容拼接为Python风格的注释
            for match in matches:
                comment_str += f"{tab} * {match}\n"
            comment_str += f'{tab} */\n'
            return comment_str

    def parse_comment_line(self, comment):
        if comment is None or comment == '':
            return ''

        comment_lines = comment.split('\n')
        if len(comment_lines) == 1:
            return f" //{comment}"
        else:
            raise SyntaxError("不支持的注释方式")

    def parse_value(self, value):
        return value['value'] if value['is_number'] else f'"{value["value"]}"'

    def parse_const(self, curr_module, ksf_const: KsfConst):
        return f"{self.parse_comment_above(ksf_const.comment)}{self.curr_tab}{self.parse_type(curr_module, ksf_const.value_type)} {ksf_const.name} = {self.parse_value(ksf_const.value)};\n\n"

    def parse_enum_member(self, ksf_enum_member: KsfEnumMember):
        return f"{self.curr_tab}{ksf_enum_member.name} = {ksf_enum_member.value}"

    def parse_enum_to_str(self, ksf_enum_member: KsfEnumMember):
        return f"{self.curr_tab}    case {ksf_enum_member.name}: return \"{ksf_enum_member.name}\";"

    def parse_str_to_enum(self, ksf_enum_member: KsfEnumMember):
        return f"{self.curr_tab}if (s == \"{ksf_enum_member.name}\") {{ e = {ksf_enum_member.name}; return 0; }}"

    def parse_default_var(self, value_type, default):
        if default is None:
            if value_type['name'] == 'int':
                return '0'
            elif value_type['name'] == 'float':
                return '0.0'
            elif value_type['name'] == 'double':
                return '0.0'
            elif value_type['name'] == 'bool':
                return 'true'
            else:
                return None

        # 特殊处理一下bool型
        if default['is_bool']:
            return f'{"true" if default["value"] else "false"}'
        elif default['is_number']:
            return f'{default["value"]}'
        else:
            return f'"{default["value"]}"'

    def parse_enum(self, curr_module, ksf_enum: KsfEnum):
        enum_str = f"{self.parse_comment_above(ksf_enum.comment)}{self.curr_tab}enum {ksf_enum.name} \n{{\n"
        self.inc_tab()
        enum_member_str_list = []
        enum_member_tostr = []
        enum_member_strto = []
        for m in ksf_enum.member:
            enum_member_str_list.append(self.parse_enum_member(m))
            enum_member_tostr.append(self.parse_enum_to_str(m))
            enum_member_strto.append(self.parse_str_to_enum(m))

        self.del_tab()

        enum_str += ',\n'.join(enum_member_str_list)

        enum_str += f"\n}};\n\n"

        etos_member = '\n'.join(enum_member_tostr)
        enum_str += f"""\
inline std::string etos(const {ksf_enum.name} &e) {{
    switch (e) {{
{etos_member}
        default: return "";
    }}
}}

"""

        stoe_member = '\n'.join(enum_member_strto)
        enum_str += f"""\
inline int stoe(const std::string &s, {ksf_enum.name} &e) {{
{stoe_member}
}}

"""
        return enum_str

    def parse_variable(self, curr_module, ksf_var: KsfField):
        default_var = self.parse_default_var(ksf_var.value_type, ksf_var.default)
        return f"{self.parse_type(curr_module, ksf_var.value_type)} \n{ksf_var.name}" \
               f"{' = ' + default_var if default_var is not None else ''}; " \
               f"{self.parse_comment_line(ksf_var.comment)}\n\n"
        pass

    def add_lines(self, lines, tab=None):
        if tab is None:
            tab = self.curr_tab

        return f"\n".join(tab + line for line in lines.split("\n")[:-1]) + '\n'

    def parse_resetDefault(self, curr_module, ksf_struct: KsfStruct):
        return self.add_lines(f'''\
/**
 * 重新赋值为初始构造的结构
 */
void resetDefault() {{
    *this = {ksf_struct.name}{{}}; 
}}\n''')

    def parse_variable_writeTo(self, ksf_field: KsfField):
        value_type = ksf_field.value_type
        if self.with_check_default:
            if value_type['type'] == 'native':
                if value_type['name'] != 'bool':
                    if value_type['name'] == 'float' or value_type['name'] == 'double':
                        return f"""if (!ksf::KS_Common::equal({ksf_field.name}, {self.parse_default_var(value_type, ksf_field.default)}{value_type['name'][0]})) {{_os.write({ksf_field.name}, {ksf_field.tag});}}"""
                    return f"""if (!{ksf_field.name} != {self.parse_default_var(value_type, ksf_field.default)}) {{_os.write({ksf_field.name}, {ksf_field.tag});}}"""
                else:
                    if ksf_field.default is None or ksf_field.default['value']:
                        return f"""if (!{ksf_field.name}) {{_os.write({ksf_field.name}, {ksf_field.tag});}}"""
                    else:
                        return f"""if ({ksf_field.name}) {{_os.write({ksf_field.name}, {ksf_field.tag});}}"""
            elif value_type['type'] == 'class':
                # 如果是枚举类型
                if value_type['name'] in self.ast.enums:
                    return f"_os.write((int32_t){ksf_field.name}, {ksf_field.tag});"
                else:
                    return f"_os.write({ksf_field.name}, {ksf_field.tag});"
            elif value_type['type'] == 'vector':
                return f"""if (!{ksf_field.name}.empty()) {{_os.write({ksf_field.name}, {ksf_field.tag});}}"""
            elif value_type['type'] == 'map':
                return f"""if (!{ksf_field.name}.empty()) {{_os.write({ksf_field.name}, {ksf_field.tag});}}"""
            else:
                raise SyntaxError(f"不能被解析的字段类型[{value_type}]")
        else:
            return f"_os.write({ksf_field.name}, {ksf_field.tag});"

    def parse_writeTo(self, curr_module, ksf_struct: KsfStruct):
        var_list = ""
        for var in ksf_struct.variable:
            var_list += self.parse_variable_writeTo(ksf_struct.variable[var]) + '\n'

        return self.add_lines(f'''\
/**
 * 序列化为二进制流
 */
template<typename WriterT>
void writeTo(ksf::KsfOutputStream<WriterT>& _os) const {{
{self.add_lines(var_list, '    ')}
}}\n''')

    def parse_variable_writeToJson(self, curr_module, ksf_field: KsfField):
        return f"""p->value["{ksf_field.name}"] = ksf::JsonOutput::writeJson({ksf_field.name});"""

    def parse_writeToJson(self, curr_module, ksf_struct: KsfStruct):
        var_list = ""
        for var in ksf_struct.variable:
            var_list += self.parse_variable_writeToJson(curr_module, ksf_struct.variable[var]) + '\n'

        return self.add_lines(f'''\
/**
 * 序列化为Json
 */
ksf::JsonValueObjPtr writeToJson() const {{
    ksf::JsonValueObjPtr p = new ksf::JsonValueObj();
{self.add_lines(var_list, '    ')}
}}

/**
 * 序列化为Json字符串
 */
std::string writeToJsonString() const {{
    return ksf::KS_Json::writeValue(writeToJson());
}}\n''')

    def parse_variable_readFrom(self, curr_module, ksf_field: KsfField):
        value_type = ksf_field.value_type
        if value_type['type'] == 'native':
            return f"""_is.read({ksf_field.name}, {ksf_field.tag}, false);"""
        elif value_type['type'] == 'class':
            return f"""_is.read({ksf_field.name}, {ksf_field.tag}, false);"""
        elif value_type['type'] == 'vector':
            return f"""_is.read({ksf_field.name}, {ksf_field.tag}, false);"""
        elif value_type['type'] == 'map':
            return f"""_is.read({ksf_field.name}, {ksf_field.tag}, false);"""
        else:
            raise SyntaxError(f"不能被解析的字段类型[{value_type}]")

    def parse_readFrom(self, curr_module, ksf_struct: KsfStruct):
        var_list = ""
        for var in ksf_struct.variable:
            var_list += self.parse_variable_readFrom(curr_module, ksf_struct.variable[var]) + '\n'

        return self.add_lines(f'''\
/**
  * 二进制反序列化为结构体
  */
template<typename ReaderT>
void readFrom(ksf::KsfInputStream<ReaderT>& _is)
{{
    resetDefault();
{self.add_lines(var_list, '    ')}
}}\n''')

    def parse_variable_readFromJson(self, curr_module, ksf_field: KsfField):
        return f"""ksf::JsonInput::readJson(c,pObj->value["{ksf_field.name}"], false);"""

    def parse_readFromJson(self, curr_module, ksf_struct: KsfStruct):
        var_list = ""
        for var in ksf_struct.variable:
            var_list += self.parse_variable_readFromJson(curr_module, ksf_struct.variable[var]) + '\n'

        return self.add_lines(f'''\
/**
 * Json反序列化为结构体
 */
ksf::JsonValueObjPtr writeToJson() const {{
    resetDefault();
    if(NULL == p.get() || p->getType() != ksf::eJsonTypeObj)
    {{
        char s[128];
        snprintf(s, sizeof(s), "read 'struct' type mismatch, get type: %d.", (p.get() ? p->getType() : 0));
        throw ksf::KS_Json_Exception(s);
    }}

    ksf::JsonValueObjPtr pObj=ksf::JsonValueObjPtr::dynamicCast(p);
{self.add_lines(var_list, '    ')}
}}

/**
 * Json字符串反序列化为结构体
 */
void readFromJsonString(const std::string &str) {{
    readFromJson(ksf::KS_Json::getValue(str));
}}
\n''')

    def parse_display(self, curr_module, ksf_struct: KsfStruct):
        var_list = ""
        for var in ksf_struct.variable:
            field = ksf_struct.variable[var]
            if field.value_type['type'] == "class" and self.get_type_id(curr_module,
                                                                        field.value_type) in self.ast.enums:
                var_list += f'_ds.display(static_cast<int32_t>({ksf_struct.variable[var].name}),"{ksf_struct.variable[var].name}");\n'
            else:
                var_list += f'_ds.display({ksf_struct.variable[var].name},"{ksf_struct.variable[var].name}");\n'

        return self.add_lines(f'''\
/**
 * 打印
 */
std::ostream& display(std::ostream& _os, int _level=0) const
{{
    ksf::KsfDisplayer _ds(_os, _level);
{self.add_lines(var_list, '    ')}
    return _os;
}}\n''')

    def parse_displaySimple(self, curr_module, ksf_struct: KsfStruct):
        var_list = ""
        for var in ksf_struct.variable:
            field = ksf_struct.variable[var]
            if field.value_type['type'] == "class" and self.get_type_id(curr_module,
                                                                        field.value_type) in self.ast.enums:
                var_list += f'_ds.displaySimple(static_cast<int32_t>({ksf_struct.variable[var].name}), true");\n'
            else:
                var_list += f'_ds.displaySimple({ksf_struct.variable[var].name}, true);\n'

        return self.add_lines(f'''\
/**
 * 简单打印
 */
std::ostream& displaySimple(std::ostream& _os, int _level=0) const
{{
    ksf::KsfDisplayer _ds(_os, _level);
{self.add_lines(var_list, '    ')}
    return _os;
}}\n''')

    def parse_equal(self, curr_module, ksf_struct: KsfStruct):
        var_list = []
        for var in ksf_struct.variable:
            field = ksf_struct.variable[var]
            if field.value_type['type'] == "native" and field.value_type['name'] in {'float', 'double'}:
                var_list.append(f'ksf::KS_Common::equal(l.{field.name}, r.{field.name})')
            else:
                var_list.append(f'l.{field.name} == r.{field.name}')

        in_list = ' && '
        var_eq = in_list.join(var_list)
        return self.add_lines(f'''\
inline bool operator==(const JsonData &l, const JsonData &r) {{
    return {var_eq};
}}\n''')

    def parse_struct(self, curr_module, ksf_struct: KsfStruct):
        struct_str = f"{self.parse_comment_above(ksf_struct.comment)}"  # 注释(可能没有)

        # struct XXX : public ksf::KsfStructBase
        # {
        struct_str += self.add_lines(f"struct {ksf_struct.name} : public ksf::KsfStructBase\n{{\n")

        # public:
        struct_str += self.add_lines(f"public:\n")
        self.inc_tab()
        #     static std::string className()
        #     {
        #         return "xx.xxx";
        #     }
        #     static std::string MD5()
        #     {
        #         return "a123123123213";
        #     }
        struct_str += self.add_lines(f'''\
static std::string className() {{
    return "{ksf_struct.module}.{ksf_struct.name}";
}}

static std::string MD5() {{
    return "{md5(ksf_struct.id.encode('utf-8')).hexdigest()}";
}}\n''')
        self.del_tab()
        # public:
        struct_str += '\n'
        struct_str += self.add_lines(f"public:\n")

        self.inc_tab()
        # 解析字段
        #     std::string appname;
        #     std::string servername;
        #     std::string sFilename;
        #     std::string sFormat;
        #     std::string setdivision;
        #     ksf::Bool bHasSufix;
        #     ksf::Bool bHasAppNamePrefix;
        #     ksf::Bool bHasSquareBracket;
        #     std::string sConcatStr;
        #     std::string sSepar;
        #     std::string sLogType;
        for var in ksf_struct.variable:
            struct_str += self.add_lines(self.parse_variable(curr_module, ksf_struct.variable[var]))

        self.del_tab()
        # public:
        struct_str += '\n'
        struct_str += self.add_lines(f"public:\n")
        self.inc_tab()
        struct_str += self.parse_resetDefault(curr_module, ksf_struct)  # resetDefault

        struct_str += '\n'
        struct_str += self.parse_writeTo(curr_module, ksf_struct)  # writeTo

        struct_str += '\n'
        struct_str += self.parse_readFrom(curr_module, ksf_struct)  # readFrom

        if self.with_json:
            struct_str += '\n'
            struct_str += self.parse_writeToJson(curr_module, ksf_struct)  # writeToJson

            struct_str += '\n'
            struct_str += self.parse_readFromJson(curr_module, ksf_struct)  # readFromJson

        struct_str += '\n'
        struct_str += self.parse_display(curr_module, ksf_struct)  # display

        struct_str += '\n'
        struct_str += self.parse_displaySimple(curr_module, ksf_struct)  # displaySimple

        self.del_tab()
        # };
        struct_str += f"{self.curr_tab}}};\n"

        # inline bool operator==(const JsonData&l, const JsonData&r)
        struct_str += '\n'
        # operator==
        struct_str += self.parse_equal(curr_module, ksf_struct)

        struct_str += '\n'
        # operator!=
        struct_str += f"""\
inline bool operator!=(const JsonData &l, const JsonData &r) {{
    return !(l == r);
}}"""

        if self.with_json:

            struct_str += '\n'
            # operator<<
            struct_str += f"""\
inline std::ostream &operator<<(std::ostream &os, const {ksf_struct.name} &r) {{
    os << r.writeToJsonString();
    return os;
}}"""

            struct_str += '\n'
            # operator>>
            struct_str += f"""\
inline std::istream &operator>>(std::istream &is, {ksf_struct.name} &l) {{
    std::istreambuf_iterator<char> eos;
    std::string s(std::istreambuf_iterator<char>(is), eos);
    l.readFromJsonString(s);
    return is;
}}"""
        return struct_str

    def parse_InterfacePrxCallBack(self, curr_module, ksf_interface: KsfInterface):
        prx_str = f"""\
class {ksf_interface.name}PrxCallback : public ksf::AgentCallback
{{
public:
    virtual ~{ksf_interface.name}PrxCallback() = default;
    
public:
\n"""

        for item in ksf_interface.operator:
            operator = ksf_interface.operator[item]

            def parse_output_vars(with_type=True):
                def parse_output_var(value_type, name, with_type):
                    if value_type['name'] != 'void':
                        if not self.is_movable_type(curr_module, value_type):
                            return f"{self.parse_type(curr_module, value_type)} {name}" if with_type else name
                        elif self.with_param_rvalue_ref:
                            return f"{self.parse_type(curr_module, value_type)} &&{name}" if with_type else f"std::move({name})"
                        else:
                            return f"const {self.parse_type(curr_module, value_type)} &{name}" if with_type else name
                vars_list = []
                if operator.return_type['name'] != 'void':
                    vars_list.append(parse_output_var(operator.return_type, '_ret', with_type))

                for item in operator.output:
                    vars_list.append(parse_output_var(operator.output[item].value_type, operator.output[item].name, with_type))

                return ', '.join(vars_list)

            prx_str += f"""\
    virtual void callback_{operator.name}({parse_output_vars(False)}) \
{{ throw std::runtime_error("callback_{operator.name} override incorrect."); }}
    
    virtual void callback_{operator.name}_exception(int32_t ret) \
{{ throw std::runtime_error("callback_{operator.name}_exception override incorrect."); }}
                
"""

        def parse_var_dispatch(var, name, index):
            if var['type'] == 'void':
                return ""

            return f"""\
                {self.parse_type(curr_module, var)} {name};
                _is.read({name}, {index}, true);\n\n"""

        def parse_func_dispatch():
            """处理函数的分发"""
            func_case_str = ""
            index = 0
            for item in ksf_interface.operator:
                operator = ksf_interface.operator[item]
                func_case_str += f"""
            case {index}: {{
                if (msg->response->iRet != ksf::KSFSERVERSUCCESS) {{
                    callback_{operator.name}_exception(msg->response->iRet);
                    return msg->response->iRet;
                }}
                
                ksf::KsfInputStream<ksf::BufferReader> _is;
                
                _is.setBuffer(msg->response->sBuffer);\n\n"""

                func_case_str += parse_var_dispatch(operator.return_type, '_ret', 0)

                for item in operator.output:
                    var = operator.output[item]
                    func_case_str += parse_var_dispatch(var.value_type, var.name, var.index)

                func_case_str += f"""\
                
                ksf::CallbackThreadData *pCbtd = ksf::CallbackThreadData::getData();
                assert(pCbtd != NULL);
                
                pCbtd->setResponseContext(msg->response->context);
                
                callback_{operator.name}({parse_output_vars(False)});
                
                pCbtd->delResponseContext();
                
                return ksf::KSFSERVERSUCCESS;
            }}"""

                index += 1

            return func_case_str

        # 函数列表
        prx_str += f"""\
public:
    virtual const std::map<std::string, std::string> &getResponseContext() const {{
        ksf::CallbackThreadData *pCbtd = ksf::CallbackThreadData::getData();
        assert(pCbtd != NULL);
        
        if (!pCbtd->getContextValid()) {{
            throw ksf::KS_Exception("cann't get response context");
        }}
        return pCbtd->getResponseContext();
    }}\n\n"""

        prx_str += f"""\
public:
    virtual int onDispatch(ksf::ReqMessagePtr _msg_)
    {{
        static std::string __{ksf_interface.name}_all[] = {{"{'", "'.join(ksf_interface.operator.keys())}"}};
                                            
        std::pair<std::string *, std::string *> r = equal_range(__{ksf_interface.name}_all, __{ksf_interface.name}_all + 9, std::string(_msg_->request.sFuncName));
        
        if (r.first == r.second) {{
            return ksf::KSFSERVERNOFUNCERR;
        }}
        
        switch (r.first - __{ksf_interface.name}_all) {{
{parse_func_dispatch()}
        }} //end switch
    }} //end onDispatch"""

        prx_str += f"""}}; //end {ksf_interface.name}PrxCallback

using {ksf_interface.name}PrxCallbackPtr = ksf::KS_AutoPtr<{ksf_interface.name}PrxCallback>;\n\n"""

        return prx_str

    def parse_InterfacePrxCallbackPromise(self, curr_module, ksf_interface: KsfInterface):
        parsed_str = f"""\
class {ksf_interface.name}PrxCallbackPromise: public ksf::ServantProxyCallback
{{
public:
    virtual ~{ksf_interface.name}PrxCallbackPromise() = default;
\n"""

        for item in ksf_interface.operator:
            operator = ksf_interface.operator[item]
            parsed_str += f"""\
public:
    struct Promise{operator.name}: virtual public KS_HandleBase
    {{
    public:
        std::map<std::string, std::string> _mRspContext;\n"""

            if operator.return_type['name'] != 'void':
                parsed_str += f"        {self.parse_type(curr_module, operator.return_type)} _ret;\n"

            for item in operator.output:
                var = operator.output[item]
                parsed_str += f"        {self.parse_type(curr_module, var.value_type)} {var.name};\n"

            parsed_str += f"""\
    }};
        
    using Promise{operator.name}Ptr = ksf::KS_AutoPtr<{ksf_interface.name}PrxCallbackPromise::Promise{operator.name}>;
    
    {ksf_interface.name}PrxCallbackPromise(const ksf::Promise<{ksf_interface.name}PrxCallbackPromise::Promise{operator.name}Ptr> &promise)
    : _promise_{operator.name}(promise) {{}}
    
    virtual void callback_{operator.name}(const {ksf_interface.name}PrxCallbackPromise::Promise{operator.name}Ptr &ptr) {{
        _promise_{operator.name}.setValue(ptr);
    }}
    
    virtual void callback_{operator.name}_exception(ksf::Int32 ret)
    {{
        std::stringstream oss;
        oss << "Function:{operator.name}_exception|Ret:";
        oss << ret;
        _promise_{operator.name}.setException(ksf::copyException(oss.str(), ret));
    }}
    
    protected:
        ksf::Promise<{ksf_interface.name}PrxCallbackPromise::Promise{operator.name}Ptr > _promise_{operator.name};\n\n"""

            def parse_var_dispatch(var, name, index):
                if var['type'] == 'void':
                    return ""

                return f"""\
                    _is.read(ptr->{name}, {index}, true);\n"""

            def parse_func_dispatch():
                """处理函数的分发"""
                func_case_str = ""
                index = 0
                for item in ksf_interface.operator:
                    operator = ksf_interface.operator[item]
                    func_case_str += f"""
            case {index}: {{
                if (msg->response->iRet != ksf::KSFSERVERSUCCESS) {{
                    callback_{operator.name}_exception(msg->response->iRet);
                    return msg->response->iRet;
                }}

                ksf::KsfInputStream<ksf::BufferReader> _is;
                _is.setBuffer(msg->response->sBuffer);
                {ksf_interface.name}PrxCallbackPromise::PromisetestConHashPtr ptr = new {ksf_interface.name}PrxCallbackPromise::PromisetestConHash();
                
                try {{\n"""
                    func_case_list = []
                    ret = parse_var_dispatch(operator.return_type, '_ret', 0)
                    if ret != "":
                        func_case_list.append(ret)

                    for item2 in operator.output:
                        var2 = operator.output[item2]
                        func_case_list.append(parse_var_dispatch(var2.value_type, var2.name, var2.index))

                    func_case_str += ''.join(func_case_list)

                    func_case_str += f"""\
                }}
                catch (std::exception &ex) {{
                    callback_{operator.name}_exception(ksf::KSFCLIENTDECODEERR);
                    return ksf::KSFCLIENTDECODEERR;
                }}
                catch (...) {{
                    callback_{operator.name}_exception(ksf::KSFCLIENTDECODEERR);
                    return ksf::KSFCLIENTDECODEERR;
                }}
                
                ptr->_mRspContext = _msg_->response->context;
                callback_{operator.name}(ptr);
                return ksf::KSFSERVERSUCCESS;
            }}\n"""

                    index += 1

                return func_case_str

            parsed_str += f"""\
public:
    virtual int onDispatch(ksf::ReqMessagePtr _msg_)
    {{
        static std::string __{ksf_interface.name}_all[] = {{"{'", "'.join(ksf_interface.operator.keys())}"}};

        std::pair<std::string *, std::string *> r = equal_range(__{ksf_interface.name}_all, __{ksf_interface.name}_all + 9, std::string(_msg_->request.sFuncName));

        if (r.first == r.second) {{
            return ksf::KSFSERVERNOFUNCERR;
        }}

        switch (r.first - __{ksf_interface.name}_all) {{
{parse_func_dispatch()}
        }} //end switch
    }} //end onDispatch"""

        parsed_str += f"\n}}; //end {ksf_interface.name}PrxCallbackPromise\n\n" \
                      f"using {ksf_interface.name}PrxCallbackPromisePtr = ksf::KS_AutoPtr<{ksf_interface.name}PrxCallbackPromise>;\n" \
                      f"\n"
        return parsed_str

    def parse_InterfaceCoroPrxCallback(self, curr_module, ksf_interface: KsfInterface):
        def parse_var_dispatch(var, name, index):
            if var['type'] == 'void':
                return ""

            return f"""\
                    {self.parse_type(curr_module, var)} {name};
                    _is.read({name}, {index}, true);\n\n"""

        def parse_func_dispatch():
            """处理函数的分发"""
            func_case_str = ""
            index = 0
            for item in ksf_interface.operator:
                operator = ksf_interface.operator[item]

                def parse_output_vars(with_type=True):
                    def parse_output_var(value_type, name, with_type):
                        if value_type['name'] != 'void':
                            if not self.is_movable_type(curr_module, value_type):
                                return f"{self.parse_type(curr_module, value_type)} {name}" if with_type else name
                            elif self.with_param_rvalue_ref:
                                return f"{self.parse_type(curr_module, value_type)} &&{name}" if with_type else f"std::move({name})"
                            else:
                                return f"const {self.parse_type(curr_module, value_type)} &{name}" if with_type else name

                    vars_list = []
                    if operator.return_type['name'] != 'void':
                        vars_list.append(parse_output_var(operator.return_type, '_ret', with_type))

                    for item in operator.output:
                        vars_list.append(
                            parse_output_var(operator.output[item].value_type, operator.output[item].name, with_type))

                    return ', '.join(vars_list)

                func_case_str += f"""
            case {index}: {{
                if (msg->response->iRet != ksf::KSFSERVERSUCCESS) {{
                    callback_{operator.name}_exception(msg->response->iRet);
                    return msg->response->iRet;
                }}
    
                ksf::KsfInputStream<ksf::BufferReader> _is;
                _is.setBuffer(msg->response->sBuffer);
    
                try {{\n"""
                func_case_list = []
                ret = parse_var_dispatch(operator.return_type, '_ret', 0)
                if ret != "":
                    func_case_list.append(ret)

                for item2 in operator.output:
                    var2 = operator.output[item2]
                    func_case_list.append(parse_var_dispatch(var2.value_type, var2.name, var2.index))

                func_case_str += ''.join(func_case_list)

                func_case_str += f"""\
                    callback_{operator.name}({parse_output_vars(with_type=False)});
                }} catch (std::exception &ex) {{
                    callback_{operator.name}_exception(ksf::KSFCLIENTDECODEERR);
                    return ksf::KSFCLIENTDECODEERR;
                }} catch (...) {{
                    callback_{operator.name}_exception(ksf::KSFCLIENTDECODEERR);
                    return ksf::KSFCLIENTDECODEERR;
                }}
    
                return ksf::KSFSERVERSUCCESS;
            }}\n"""

                index += 1

            return func_case_str

        parsed_str = f"""\
class {ksf_interface.name}CoroPrxCallback : public {ksf_interface.name}PrxCallback
{{
public:
    virtual ~{ksf_interface.name}CoroPrxCallback() == default;

public:
    virtual const std::map<std::string, std::string> &getResponseContext() const {{ return _mRspContext; }}
    
    virtual void setResponseContext(const std::map<std::string, std::string> &mContext) {{ _mRspContext = mContext; }}
"""
        parsed_str += f"""\
public:
    virtual int onDispatch(ksf::ReqMessagePtr _msg_)
    {{
        static std::string __{ksf_interface.name}_all[] = {{"{'", "'.join(ksf_interface.operator.keys())}"}};

        std::pair<std::string *, std::string *> r = equal_range(__{ksf_interface.name}_all, __{ksf_interface.name}_all + 9, std::string(_msg_->request.sFuncName));

        if (r.first == r.second) {{
            return ksf::KSFSERVERNOFUNCERR;
        }}

        switch (r.first - __{ksf_interface.name}_all) {{
{parse_func_dispatch()}
        }} //end switch
        
        return ksf::KSFSERVERNOFUNCERR;
    }} //end onDispatch

protected:
    std::map<std::string, std::string> _mRspContext;
}}; //end {ksf_interface.name}CoroPrxCallback

typedef ksf::KS_AutoPtr<{ksf_interface.name}CoroPrxCallback> {ksf_interface.name}CoroPrxCallbackPtr;\n"""
        return parsed_str

    def parse_InterfaceProxy(self, curr_module, ksf_interface: KsfInterface):
        def parse_operators():
            parsed_oper_str = ""
            for item in ksf_interface.operator:
                operator = ksf_interface.operator[item]

                def has_return():
                    return operator.return_type['name'] != 'void'

                def parse_input_vars():
                    parsed_list = []
                    for name in operator.input:
                        var = operator.input[name]
                        parsed_list.append(f"""\
        _os.write({name}, {var.index});\n""")
                    return parsed_list

                def parse_output_vars():
                    parsed_list = []
                    if operator.return_type['name'] != 'void':
                        parsed_list.append(f"""\
        {self.parse_type(curr_module, operator.return_type)} _ret;
        _is.read(_ret, 0, true);\n""")

                    for name in operator.output:
                        var = operator.output[name]
                        parsed_list.append(f"""\
        _is.read({name}, {var.index}, true);\n""")
                    return parsed_list

                def parse_all_vars(with_output=False):
                    parsed_list = []
                    for var_name, is_output in operator.ordered_var:
                        if is_output:
                            if with_output:
                                parsed_list.append(
                                    f"{self.parse_type(curr_module, operator.output[var_name].value_type)} &{var_name}, ")
                        else:
                            parsed_list.append(
                                f"const {self.parse_type(curr_module, operator.input[var_name].value_type)} &{var_name}, ")

                    return parsed_list

                parsed_oper_str += f"""\
public:
{self.parse_comment_above(operator.comment, tab='    ')}\
    {self.parse_type(curr_module, operator.return_type) if has_return() else 'void'} {operator.name}({''.join(parse_all_vars(True))}const std::map<std::string, std::string> &mapReqContext = KSF_CONTEXT(), std::map<std::string, std::string> *pResponseContext = NULL) {{
        ksf::KsfOutputStream<ksf::BufferWriterVector> _os;
{''.join(parse_input_vars())}
        std::map<std::string, std::string>   _mStatus;
        std::shared_ptr<ksf::ResponsePacket> rep = ksf_invoke(ksf::KSFNORMAL, "{operator.name}", _os, mapReqContext, _mStatus);
        if (pResponseContext) {{
            pResponseContext->swap(rep->context);
        }}
        
        ksf::KsfInputStream<ksf::BufferReader> _is;
        _is.setBuffer(rep->sBuffer);
{''.join(parse_output_vars())}
        {'return _ret;' if has_return() else ''}
    }}
    
    void async_{operator.name}({ksf_interface.name}PrxCallbackPtr callback, {''.join(parse_all_vars())}const std::map<std::string, std::string> &context = KSF_CONTEXT()) {{
        ksf::KsfOutputStream<ksf::BufferWriterVector> _os;
{''.join(parse_input_vars())}
        std::map<std::string, std::string> _mStatus;
        ksf_invoke_async(ksf::KSFNORMAL, "{operator.name}", _os, context, _mStatus, callback);
    }}
    
    ksf::Future<{ksf_interface.name}PrxCallbackPromise::Promise{operator.name}Ptr> promise_async_{operator.name}({''.join(parse_all_vars())}const std::map<std::string, std::string> &context) {{
        ksf::Promise<{ksf_interface.name}PrxCallbackPromise::Promise{operator.name}Ptr> promise;
        {ksf_interface.name}PrxCallbackPromisePtr callback = new {ksf_interface.name}PrxCallbackPromise(promise);
        
        ksf::KsfOutputStream<ksf::BufferWriterVector> _os;
{''.join(parse_input_vars())}
        std::map<std::string, std::string> _mStatus;
        ksf_invoke_async(ksf::KSFNORMAL, "{operator.name}", _os, context, _mStatus, callback);
        
        return promise.getFuture();
    }}
    
    void coro_{operator.name}({ksf_interface.name}CoroPrxCallbackPtr callback, {''.join(parse_all_vars())}const std::map<std::string, std::string> &context = KSF_CONTEXT())
    {{
        ksf::KsfOutputStream<ksf::BufferWriterVector> _os;
{''.join(parse_input_vars())}
        std::map<std::string, std::string>            _mStatus;
        ksf_invoke_async(ksf::KSFNORMAL, "{operator.name}", _os, context, _mStatus, callback, true);
    }}\n\n"""
            return parsed_oper_str

        parsed_str = f"""\
class {ksf_interface.name}Proxy : public ksf::Agent
{{
public:
    typedef std::map<std::string, std::string> KSF_CONTEXT;
    
public:
    {ksf_interface.name}Proxy* ksf_hash(uint32_t key) {{
        return ({ksf_interface.name}Proxy*)Agent::ksf_hash(key);
    }}

    {ksf_interface.name}Proxy* ksf_consistent_hash(uint32_t key) {{
        return ({ksf_interface.name}Proxy*)Agent::ksf_consistent_hash(key);
    }}

    {ksf_interface.name}Proxy* ksf_open_trace(bool traceParam = false) {{
        return ({ksf_interface.name}Proxy*)Agent::ksf_open_trace(traceParam);
    }}

    {ksf_interface.name}Proxy* ksf_set_timeout(int msecond) {{
        return ({ksf_interface.name}Proxy*)Agent::ksf_set_timeout(msecond);
    }}

    static const char* ksf_prxname() {{ return "{ksf_interface.name}Proxy"; }}
    
{parse_operators()}
}} //end {ksf_interface.name}Proxy

using {ksf_interface.name}Prx = ksf::KS_AutoPtr<{ksf_interface.name}Proxy>;\n\n"""

        return parsed_str

    def parse_InterfaceObj(self, curr_module, ksf_interface: KsfInterface):
        parsed_str = f"""\
class {ksf_interface.name} : public ksf::Servant
{{
public:
    ~{ksf_interface.name}() override = default;
    
public:
/*必须继承并实现的函数*/
"""
        parsed_async_str = ''
        parsed_push_str = ''
        dispatch_case_str = ""
        index = 0
        for item in ksf_interface.operator:
            operator = ksf_interface.operator[item]

            def has_return():
                return operator.return_type['name'] != 'void'

            def parse_invoke_vars():
                parsed_list = []
                for name, is_output in operator.ordered_var:
                    if not is_output:
                        var = operator.input[name]
                        if self.with_param_rvalue_ref and self.is_movable_type(curr_module, var.value_type):
                            parsed_list.append(f"""std::move({var.name}), """)
                        else:
                            parsed_list.append(f"""{var.name}, """)
                    else:
                        var = operator.output[name]
                        parsed_list.append(f"""{var.name}, """)

                return ''.join(parsed_list)

            def parse_output_vars(mode='ksf', tab=None):
                if tab is None:
                    tab = 0

                parsed_list = []
                if operator.return_type['name'] != 'void':
                    if mode == 'ksf':
                        parsed_list.append(f"""_os.write(_ret, 0);""")
                    elif mode == 'kup':
                        parsed_list.append(f"""_ksfAttr_.put("", _ret);""")
                        parsed_list.append(f"""_ksfAttr_.put("ksf_ret", _ret);""")
                    elif mode == 'json':
                        parsed_list.append(f"""_p->value["ksf_ret"] = ksf::JsonOutput::writeJson(_ret);""")

                for name in operator.output:
                    var = operator.output[name]
                    if mode == 'ksf':
                        parsed_list.append(f"""_os.write({name}, {var.index});""")
                    elif mode == 'kup':
                        parsed_list.append(f"""_ksfAttr_.put("{name}", {name});""")
                    elif mode == 'json':
                        parsed_list.append(f"""_p->value["{name}"] = ksf::JsonOutput::writeJson({name});""")
                return '\n'.join([' ' * (4 * tab) + line for line in parsed_list])

            def parse_all_vars(with_input=False):
                parsed_list = []
                if not with_input and has_return():
                    parsed_list.append(f", const {self.parse_type(curr_module, operator.return_type)} &_ret")

                for var_name, is_output in operator.ordered_var:
                    if is_output:
                        if with_input:
                            parsed_list.append(
                                f"{self.parse_type(curr_module, operator.output[var_name].value_type)} &{var_name}, ")
                        else:
                            parsed_list.append(
                                f", const {self.parse_type(curr_module, operator.output[var_name].value_type)} &{var_name}")
                    elif with_input:
                        if self.with_param_rvalue_ref and self.is_movable_type(curr_module, operator.input[var_name].value_type):
                            parsed_list.append(
                                f"{self.parse_type(curr_module, operator.input[var_name].value_type)} &&{var_name}, ")
                        else:
                            parsed_list.append(
                                f"const {self.parse_type(curr_module, operator.input[var_name].value_type)} &{var_name}, ")

                return parsed_list

            parsed_str += f"""\
{self.parse_comment_above(operator.comment, tab='    ')}\
    virtual {self.parse_type(curr_module, operator.return_type) if has_return() else 'void'} {operator.name}({''.join(parse_all_vars(True))}ksf::KsfCurrentPtr _current_) = 0;\n\n"""

            parsed_async_str += f"""\
    static void async_response_{operator.name}(ksf::KsfCurrentPtr _current_{''.join(parse_all_vars())})
    {{
        switch (_current_->getRequestVersion()) {{
            case ksf::KUPVERSION: {{
                ksf::kup::UniAttribute<ksf::BufferWriterVector, ksf::BufferReader> _ksfAttr_;
                _ksfAttr_.setVersion(_current_->getRequestVersion());
{parse_output_vars(mode='kup', tab=4)}
                std::vector<char> sKupResponseBuffer;
                _ksfAttr_.encode(sKupResponseBuffer);
                _current_->sendResponse(ksf::KSFSERVERSUCCESS, sKupResponseBuffer);
                break;
            }}\n"""
            if self.with_json:
                parsed_async_str += f"""\
            case ksf::JSONVERSION: {{
                ksf::JsonValueObjPtr _p = new ksf::JsonValueObj();
{parse_output_vars(mode='json', tab=4)}
                std::vector<char> sJsonResponseBuffer;
                ksf::KS_Json::writeValue(_p, sJsonResponseBuffer);
                _current_->sendResponse(ksf::KSFSERVERSUCCESS, sJsonResponseBuffer);
                break;
        }}\n"""
            parsed_async_str += f"""\
            case ksf::KSFVERSION: {{
                ksf::KsfOutputStream<ksf::BufferWriterVector> _os;
{parse_output_vars(mode='ksf', tab=4)}
                _current_->sendResponse(ksf::KSFSERVERSUCCESS, _os);
                break;
            }}
                default: {{std::runtime_error("unsupport ksf packet version");}}
        }} // end switch\n\n"""

            # 链路追踪
            if self.with_trace:
                if not self.with_json:
                    raise SyntaxError("如果需要链路追踪，需要打开生成Json序列化支持(--json)")

                parsed_async_str += f"""\
        if (_current_->isTraced()) {{
            std::string _trace_param_;
            int _trace_param_flag_ = ksf::ServantProxyThreadData::needTraceParam(ksf::ServantProxyThreadData::TraceContext::EST_SS, _current_->getTraceKey(), _rsp_len_);
            if (ksf::ServantProxyThreadData::TraceContext::ENP_NORMAL == _trace_param_flag_) {{
                ksf::JsonValueObjPtr _p_ = new ksf::JsonValueObj();
                {'_p_->value[""] = ksf::JsonOutput::writeJson(_ret);' if has_return() else ''}
                _trace_param_ = ksf::KS_Json::writeValue(_p_);
            }} else if(ksf::ServantProxyThreadData::TraceContext::ENP_OVERMAXLEN == _trace_param_flag_) {{
                _trace_param_ = "{{\"trace_param_over_max_len\":true}}";
            }}
            
            KSF_TRACE(_current_->getTraceKey(), TRACE_ANNOTATION_SS, "", ksf::ServerConfig::Application + "." + ksf::ServerConfig::ServerName, "test", 0, _trace_param_, "");
        }}\n"""
            dispatch_case_str += f"""\
            case {index}: {{
                ksf::KsfInputStream<ksf::BufferReader> _is;
                _is.setBuffer(_current->getRequestBuffer());
                switch (_current->getRequestVersion()): {{
                    case ksf::KUPVERSION: {{
                        ksf::kup::UniAttribute<ksf::BufferWriterVector, ksf::BufferReader>  _ksfAttr_;
                        _ksfAttr_.setVersion(_current->getRequestVersion());
                        _ksfAttr_.decode(_current->getRequestBuffer());
                        break;
                    }} 
                    case ksf::JSONVERSION: {{
                        ksf::JsonValueObjPtr _jsonPtr = ksf::JsonValueObjPtr::dynamicCast(ksf::KS_Json::getValue(_current->getRequestBuffer()));
                        break;
                    }}
                    default: {{}}
                }}
                
                {"ksf::Int32 _ret = " if has_return() else ""}{operator.name}({parse_invoke_vars()}_current);
                if(_current->isResponse()) {{
                    switch (_current->getRequestVersion()): {{
                        case ksf::KUPVERSION: {{
                            ksf::kup::UniAttribute<ksf::BufferWriterVector, ksf::BufferReader>  _ksfAttr_;
                            _ksfAttr_.setVersion(_current->getRequestVersion());
{parse_output_vars(mode='kup', tab=7)}
                            _ksfAttr_.encode(_sResponseBuffer);
                            break;
                        }} 
                        case ksf::JSONVERSION: {{
                            ksf::JsonValueObjPtr _p = new ksf::JsonValueObj();
{parse_output_vars(mode='json', tab=7)}
                            ksf::KS_Json::writeValue(_p, _sResponseBuffer);
                            break;
                        }} 
                        case ksf::KSFVERSION: {{
                            ksf::KsfOutputStream<ksf::BufferWriterVector> _os;
{parse_output_vars(mode='ksf', tab=7)}
                            _os.swap(_sResponseBuffer);
                            break;
                        }}
                        default: {{}}
                    }}
                }}
                return ksf::KSFSERVERSUCCESS;

            }}\n"""
            index += 1

            parsed_async_str += f"""\
    }}
\n"""
            parsed_push_str += f"""\
    static void async_response_push_{operator.name}(ksf::KsfCurrentPtr _current_{''.join(parse_all_vars())})
    {{
        {{
            ksf::KsfOutputStream<ksf::BufferWriterVector> _os;
{parse_output_vars(mode='ksf', tab=3)}
            _current_->sendPushResponse(ksf::KSFSERVERSUCCESS, "{operator.name}", _os, _context);
        }}
    }}
\n"""

        parsed_str += f"""public:
/*异步应答的封装函数*/\n\n"""
        parsed_str += parsed_async_str

        parsed_str += f"""public:
/*推送应答的封装函数*/\n\n"""

        parsed_str += parsed_push_str

        parsed_str += f"""\
public:
    virtual int onDispatch(ksf::ReqMessagePtr _msg_)
    {{
        static std::string __{ksf_interface.name}_all[] = {{"{'", "'.join(ksf_interface.operator.keys())}"}};
                                            
        std::pair<std::string *, std::string *> r = equal_range(__{ksf_interface.name}_all, __{ksf_interface.name}_all + 9, std::string(_msg_->request.sFuncName));
        
        if (r.first == r.second) {{
            return ksf::KSFSERVERNOFUNCERR;
        }}
        
        switch (r.first - __{ksf_interface.name}_all) {{
"""

        parsed_str += dispatch_case_str
        parsed_str += f"""\
        }} //end switch
    }} //end onDispatch"""

        parsed_str += f"""}}; // end {ksf_interface.name}\n\n"""
        return parsed_str

    def parse_interface(self, curr_module, ksf_interface: KsfInterface):
        interface_str = ''

        interface_str += self.add_lines(self.parse_InterfacePrxCallBack(curr_module, ksf_interface))
        interface_str += self.add_lines(self.parse_InterfacePrxCallbackPromise(curr_module, ksf_interface))
        interface_str += self.parse_InterfaceCoroPrxCallback(curr_module, ksf_interface)
        interface_str += self.parse_InterfaceProxy(curr_module, ksf_interface)
        interface_str += self.parse_InterfaceObj(curr_module, ksf_interface)

        return interface_str

    def to_file(self):
        # 生成头文件
        for inc in self.ast_in_file.includes:
            self.add_include(f'"{inc.name[:-4]}.h"')

        curr_module = None
        for ele in self.ast_in_file.elements:
            # 生成命名空间
            if curr_module is None:
                curr_module = ele.module
                self.file_str += self.curr_tab + f"namespace {ele.module} {{\n\n"
            elif curr_module != ele.module:
                curr_module = ele.module
                self.file_str += self.curr_tab + f"}} //end {curr_module}\n\n\nnamespace {ele.module} {{\n\n"

            if isinstance(ele, KsfConst):
                self.file_str += self.parse_const(curr_module, ele)
                pass
            elif isinstance(ele, KsfEnum):
                self.file_str += self.parse_enum(curr_module, ele)
                pass
            elif isinstance(ele, KsfStruct):
                self.file_str += self.parse_struct(curr_module, ele)
                pass
            elif isinstance(ele, KsfInterface) and self.with_rpc:
                self.file_str += self.parse_interface(curr_module, ele)

        self.file_str += f"}} //end {curr_module}\n\n"

        self.file_str = self.parse_header() + "\n\n" + self.file_str
        # print(self.file_str)

        with self.generated_file.open("w") as f:
            f.write(self.file_str)


def cpp_gen(files, include_dirs, destination_dir, flags=None):
    # 读取文件
    # parser_grammar = {}
    if flags is None:
        flags = defaultdict(bool)
    file_path = files

    real_inc_dirs = set()
    for inc in include_dirs:
        real_inc_dirs.add(str(Path(inc).resolve()))
    ast = generate(file_path, real_inc_dirs, flags['with_current_priority'])
    for file in file_path:
        CppGenerator(ast, file, destination_dir, **flags).to_file()


# 测试用例
if __name__ == '__main__':
    file_path = [
        'example/enum_simple.ksf',
        'example/const_definition.ksf',
        'example/struct_simple.ksf'
    ]

    include_dirs = ['../../']

    destination_dir = '../../gen'

    cpp_gen(file_path, include_dirs, destination_dir)



from setuptools import setup, find_packages

setup(
    name='ksfctl',
    version='0.1',
    keywords='ksf, client, generator, tool',
    description='a auto generator tools for ksf protocol files, and some useful tools',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'Click>=8.0.0',
    ],
    entry_points='''
        [console_scripts]
        ksfctl=ksfctl.cmd.cmd:cli
        ksf2cpp=ksfctl.cmd.cmd:cpp
    ''',
)



import json
import warnings

import ply.yacc as yacc

from ksfctl.parser.ksf_parser import *
from ksfctl.parser.tools import Graph
from ksfctl.parser.ksf_lex import KsfLexer


class KsfParser:
    def __init__(self):
        # 解析语法的准备
        self.lexer = KsfLexer()
        self.lexer.build()
        self.tokens = self.lexer.tokens
        self.parser = yacc.yacc(module=self)

    def parse(self, data):
        self.text = data
        return self.parser.parse(data)

    def p_file(self, p):
        """
        file : module_list
             | ksf_header module_list
        """
        if len(p) == 3:
            p[0] = {'include': p[1], 'module': p[2]}
        else:
            p[0] = {'include': [], 'module': p[1]}

    def p_ksf_header(self, p):
        '''ksf_header : comment_multi include_list
        '''
        p[0] = p[2]

    def p_include_list(self, p):
        '''include_list : include_list include_item
                        | include_item'''
        if len(p) == 2:
            p[0] = [p[1]]
        else:
            p[0] = p[1] + [p[2]]

    def p_include_item(self, p):
        '''include_item : INCLUDE STRING_CONSTANT'''
        p[0] = p[2]

    def p_module_name(self, p):
        """
        module_name : IDENTIFIER
        """
        p[0] = p[1]

    def p_module(self, p):
        """
        module : comment_multi MODULE module_name LEFT_BRACE element_list RIGHT_BRACE SEMICOLON comment_line
        """
        p[0] = {'name': p[3], 'comment': p[1] if p[1] is not None else p[8], 'elements': p[5]}

    def p_module_list(self, p):
        """
        module_list : module
                    | module_list module
        """
        if len(p) == 2:
            p[0] = [p[1]]
        else:
            p[1].append(p[2])
            p[0] = p[1]

    def p_return_type(self, p):
        """
        return_type : type
        """
        p[0] = p[1]

    def p_operation(self, p):
        """
        operation : comment_multi return_type operation_name LEFT_PAREN operation_param_list RIGHT_PAREN SEMICOLON comment_line
        """
        p[0] = {
            'type': 'operation',
            'name': p[3],
            'return_type': p[2],
            'variable': p[5],
            'comment': p[1] if p[1] is not None else p[8]
        }

    def p_element_list(self, p):
        """
        element_list : element
                     | element_list element
        """
        if len(p) == 2:
            p[0] = [p[1]]
        else:
            p[1].append(p[2])
            p[0] = p[1]

    def p_comment_multi(self, p):
        """
        comment_multi : COMMENT_MULTILINE
                | empty
        """
        if len(p) == 2 and isinstance(p[1], str):
            p[0] = p[1]
        else:
            p[0] = None

    def p_comment_line(self, p):
        """
        comment_line : COMMENT_LINE
                | empty
        """
        if len(p) == 2 and isinstance(p[1], str):
            p[0] = p[1]
        else:
            p[0] = None

    ####### 元素 ########
    def p_element(self, p):
        """
        element : interface
                | struct
                | enum
                | constant
                | struct_keyword
        """
        p[0] = p[1]

    ####### 接口 #######
    def p_interface(self, p):
        """
        interface : comment_multi INTERFACE interface_name LEFT_BRACE operation_list RIGHT_BRACE SEMICOLON
        """
        p[0] = {
            'element': 'interface',
            'name': p[3],
            'operator': p[5],
            'comment': p[1]
        }

    ####### 结构体 ########
    def p_struct(self, p):
        """
        struct : comment_multi STRUCT struct_name LEFT_BRACE variable_list RIGHT_BRACE SEMICOLON
        """
        p[0] = {
            'element': 'struct',
            'name': p[3],
            'variable': p[5],
            'comment': p[1]
        }

    ####### 常量 ########
    def p_constant(self, p):
        """
        constant : comment_multi CONST type IDENTIFIER EQUALS value SEMICOLON comment_line
        """
        p[0] = {
            'element': 'constant',
            'type': p[3],
            'name': p[4],
            'value': p[6],
            'comment': p[1] if p[1] is not None else p[8],
        }

    ####### 枚举 ########
    def p_enum(self, p):
        """
        enum : comment_multi ENUM enum_name LEFT_BRACE enum_items RIGHT_BRACE SEMICOLON
        """
        p[0] = {
            'element': 'enum',
            'name': p[3],
            'type': 'int32',
            'member': p[5],
            'comment': p[1]
        }

    ####### 结构体进行比较时需要的字段关联 #######
    def p_struct_keyword(self, p):
        """
        struct_keyword : KEY LEFT_SQUARE IDENTIFIER COMMA key_params RIGHT_SQUARE SEMICOLON
        """
        p[0] = {
            'element': 'key',
            'struct': p[3],
            'args': p[5]
        }

    ######### 各类list #########
    #### 枚举值列表 ####
    def p_enum_items(self, p):
        """
        enum_items : enum_item
                   | enum_items COMMA enum_item
        """
        if len(p) == 2:
            if 'value' not in p[1]:
                p[1]['value'] = 0
            p[0] = [p[1]]
        else:
            if 'value' not in p[3]:
                p[3]['value'] = p[1][-1]['value'] + 1

            p[0] = p[1] + [p[3]]

    def p_operation_list(self, p):
        """
        operation_list : operation
                       | operation_list operation
        """
        if len(p) == 2:
            p[0] = [p[1]]
        else:
            p[1].append(p[2])
            p[0] = p[1]

    def p_operation_name(self, p):
        """
        operation_name : IDENTIFIER
        """
        p[0] = p[1]

    def p_operation_param_list(self, p):
        """
        operation_param_list : operation_param
                             | operation_param_list COMMA operation_param
        """
        if len(p) == 2:
            p[0] = [p[1]]
        else:
            p[0] = p[1] + [p[3]]

    def p_operation_output(self, p):
        """
        operation_output : OUTPUT
                        | empty
        """
        if p[1] is None:
            p[0] = False
        else:
            p[0] = True

    def p_operation_param(self, p):
        """
        operation_param : operation_output type IDENTIFIER
        """
        p[0] = {'is_output': p[1],
                'value_type': p[2],
                'name': p[3]}

    def p_interface_name(self, p):
        """
        interface_name : IDENTIFIER
        """
        p[0] = p[1]

    def p_struct_name(self, p):
        """
        struct_name : IDENTIFIER
        """
        p[0] = p[1]

    def p_variable_list(self, p):
        """
        variable_list : variable
                       | variable_list variable
        """
        if len(p) == 2:
            p[0] = [p[1]]
        else:
            p[1].append(p[2])
            p[0] = p[1]

    def p_variable_name(self, p):
        """
        variable_name : IDENTIFIER
        """
        p[0] = p[1]

    def p_enum_name(self, p):
        """
        enum_name : IDENTIFIER
        """
        p[0] = p[1]

    def p_enum_item(self, p):
        """
        enum_item : IDENTIFIER EQUALS number_value comment_line
                  | IDENTIFIER comment_line
        """
        if len(p) == 4:
            p[0] = {
                'name': p[1],
                'value': int(p[3], 0),
                'comment': p[4]
            }
        else:
            p[0] = {
                'name': p[1],
                'comment': p[2]
            }

    ######## 类型定义 ########
    def p_unsigned_type(self, p):
        """
        unsigned_type : UNSIGNED INT
                        | UNSIGNED SHORT
                        | UNSIGNED LONG
        """
        if p[2] == 'long':
            p[0] = {'element': 'type', 'type': 'native', 'name': 'int', 'bit': 64, 'unsigned': True}
        elif p[2] == 'short':
            p[0] = {'element': 'type', 'type': 'native', 'name': 'int', 'bit': 16, 'unsigned': True}
        elif p[2] == 'int':
            p[0] = {'element': 'type', 'type': 'native', 'name': 'int', 'bit': 32, 'unsigned': True}

    def p_primitive_type(self, p):
        """
        primitive_type : INT
                       | SHORT
                       | LONG
                       | FLOAT
                       | DOUBLE
                       | CHAR
                       | BOOL
                       | STRING
                       | unsigned_type
        """
        if isinstance(p[1], str):
            if p[1] == 'int':
                bit = 32
                p[0] = {'element': 'type', 'type': 'native', 'name': 'int', 'bit': bit, 'unsigned': False}
            elif p[1] == 'short':
                bit = 16
                p[0] = {'element': 'type', 'type': 'native', 'name': 'int', 'bit': bit, 'unsigned': False}
            elif p[1] == 'long':
                bit = 64
                p[0] = {'element': 'type', 'type': 'native', 'name': 'int', 'bit': bit, 'unsigned': False}
            elif p[1] == 'float':
                bit = 32
                p[0] = {'element': 'type', 'type': 'native', 'name': 'float', 'bit': bit, 'unsigned': False}
            elif p[1] == 'double':
                bit = 64
                p[0] = {'element': 'type', 'type': 'native', 'name': 'double', 'bit': bit, 'unsigned': False}
            elif p[1] == 'byte':
                bit = 8
                p[0] = {'element': 'type', 'type': 'native', 'name': 'int', 'bit': bit, 'unsigned': False}
            else:
                p[0] = {'element': 'type', 'type': 'native', 'name': p[1]}
        else:
            p[0] = p[1]

    def p_struct_type(self, p):
        """
        struct_type : struct_name
        """
        v = p[1].split('::')
        if len(v) == 1:
            module = None
            name = v[0]
        else:
            module = v[0]
            name = v[1]

        p[0] = {
            'element': 'type',
            'type': 'class',  # 可能为struct，也可能为enum，需后面进行判定
            'name': name,
            'module': module,
        }

    def p_map_type(self, p):
        """
        map_type : MAP LEFT_ANGLE_BRACKET type COMMA type RIGHT_ANGLE_BRACKET
             | MAP LEFT_SQUARE IDENTIFIER RIGHT_SQUARE LEFT_ANGLE_BRACKET type COMMA type RIGHT_ANGLE_BRACKET
        """
        if len(p) == 7:
            p[0] = {
                'element': 'type',
                'type': 'map',
                'is_ordered': True,
                'is_hashed': False,
                'is_unique_member': True,
                'name': 'map<' + p[3]['name'] + ', ' + p[5]['name'] + '>',
                'key_type': p[3],
                'value_type': p[5]
            }
        else:
            if p[3] == 'hashmap':
                p[0] = {
                    'element': 'type',
                    'type': 'map',
                    'is_ordered': False,
                    'is_hashed': True,
                    'is_unique_member': True,
                    'name': 'hashmap<' + p[6]['name'] + ', ' + p[8]['name'] + '>',
                    'key_type': p[6],
                    'value_type': p[8]
                }
            else:
                raise Exception(f"Syntax error at token {p[1]}")

    def p_vector_type(self, p):
        """
        vector_type : VECTOR LEFT_ANGLE_BRACKET type RIGHT_ANGLE_BRACKET
             | VECTOR LEFT_SQUARE IDENTIFIER RIGHT_SQUARE LEFT_ANGLE_BRACKET type RIGHT_ANGLE_BRACKET
        """
        if len(p) == 5:
            p[0] = {
                'element': 'type',
                'type': 'vector',
                'is_ordered': True,
                'is_hashed': False,
                'is_unique_member': False,
                'name': 'vector<' + p[3]['name'] + '>',
                'value_type': p[3]
            }
        else:
            if p[3] == 'set':
                p[0] = {
                    'element': 'type',
                    'type': 'vector',
                    'is_ordered': True,
                    'is_hashed': False,
                    'is_unique_member': True,
                    'name': 'set<' + p[6]['name'] + '>',
                    'value_type': p[6]
                }
            elif p[3] == 'hashset':
                p[0] = {
                    'element': 'type',
                    'type': 'vector',
                    'is_ordered': False,
                    'is_hashed': True,
                    'is_unique_member': True,
                    'name': 'hashset<' + p[6]['name'] + '>',
                    'value_type': p[6]
                }
            else:
                raise Exception(f"Syntax error at token {p[1]}")

    def p_array_type(self, p):
        """
        array_type : type LEFT_SQUARE RIGHT_SQUARE
        """
        p[0] = {
            'element': 'type',
            'type': 'array',
            'name': 'array<' + p[1]['name'] + '>',
            'value_type': p[1],
        }

    def p_type(self, p):
        """
        type : primitive_type
             | vector_type
             | map_type
             | array_type
             | struct_type
        """
        p[0] = p[1]

    ######## 值定义 ########
    def p_number_value(self, p):
        """
        number_value : INT_CONSTANT
        | EXPR_CONSTANT
        | RADIX_CONSTANT
        """
        p[0] = p[1]

    def p_string_value(self, p):
        """
        string_value : STRING_CONSTANT
        """
        p[0] = p[1]

    def p_bool_value(self, p):
        """
        bool_value : IDENTIFIER
        """
        if p[1] == 'true':
            p[0] = p[1]
        elif p[1] == 'false':
            p[0] = p[1]
        else:
            raise SyntaxError('布尔类型只支持(0, 1)或者(false, true)')

    def p_value(self, p):
        """
        value : string_value
              | number_value
              | bool_value
        """
        is_number = True if p.slice[1].type == 'number_value' else False
        is_bool = True if p.slice[1].type == 'bool_value' else False
        p[0] = {'value': p[1], 'is_number': is_number, 'is_bool': is_bool}

    def p_ro(self, p):
        """
        ro : REQUIRED
           | OPTIONAL
        """
        if p[1] == 'require':
            p[0] = True
        elif p[1] == 'optional':
            p[0] = False
        else:
            raise Exception(f"Syntax error at token {KsfLexer.find_position(p.lexer.lexdata, p.lexer)}")

    def p_tag(self, p):
        """
        tag : number_value
        """

        def is_valid(s):
            if s.isdigit():
                num = int(s, 0)
                if 0 <= num <= 255:
                    return True
            return False

        if is_valid(p[1]):
            p[0] = p[1]
        else:
            raise Exception(
                f"Syntax error at token {p.type} ({p.value}), line {KsfLexer.find_position(p.lexer.lexdata, p.lexer)}")

    def p_variable(self, p):
        """
        variable : tag ro type variable_name SEMICOLON comment_line
        | tag ro type variable_name EQUALS value SEMICOLON comment_line
        """

        if len(p) == 7:
            variable = {'type': 'variable',
                        'tag': p[1],
                        'is_required': p[2],
                        'value_type': p[3],
                        'name': p[4],
                        'comment': p[6]}
        else:
            variable = {'type': 'variable',
                        'tag': p[1],
                        'is_required': p[2],
                        'value_type': p[3],
                        'name': p[4],
                        'default': p[6],
                        'comment': p[8]}

        # 特殊处理一下bool类型
        if 'default' in variable and variable['value_type']['name'] == 'bool':
            variable['default']['is_number'] = False
            variable['default']['is_bool'] = True
            if variable['default']['value'] == '1':
                variable['default']['value'] = True
            elif variable['default']['value'] == '0':
                variable['default']['value'] = False
            elif variable['default']['value'] == 'true':
                variable['default']['value'] = True
            elif variable['default']['value'] == 'false':
                variable['default']['value'] = False
            else:
                raise SyntaxError(f'不支持的布尔默认值[{p[6]}]，请检查')

        p[0] = variable

    def p_key_params(self, p):
        """
        key_params : IDENTIFIER
                   | key_params COMMA IDENTIFIER
        """
        if len(p) == 2:
            p[0] = [p[1]]
        else:
            p[0] = p[1] + [p[3]]

    def p_empty(self, p):
        """
        empty :
        """
        pass

    def p_error(self, p):
        if p:
            if p.type in {'COMMENT_MULTILINE', 'COMMENT_LINE'}:
                self.parser.errok()
                return
            else:
                raise SyntaxError(
                    f"Syntax error at token {p.type} ({p.value}), line {KsfLexer.find_position(p.lexer.lexdata, p.lexer)}")
        else:
            SyntaxError("Syntax error at EOF")


def pretty_print(jvalue):
    print(json.dumps(jvalue, indent=2, ensure_ascii=False))


def generate_ksf_ast(grammar, search_dirs):
    """
    检查解析出的语法数据是否合法，并生成基础ast树
    :param grammar:
    :return:
    """

    # 1. 生成头文件包含树
    g_files = Graph()  # 文件有向图
    g_files_reverse = Graph()  # 文件逆向有向图
    for file in grammar:
        curr = file
        if len(grammar[file]['include']) != 0:
            for include in grammar[file]['include']:
                # 在search路径中添加当前文件所在路径

                inc_path = Path(include)
                if inc_path is None:
                    raise FileNotFoundError(f"依赖的头文件[{include}]未找到")

                # print(f'{item}->{inc_path.name}')
                if inc_path.name == curr:
                    raise SyntaxError(f"不允许#include自身，[{curr}]存在错误的依赖")

                if inc_path.name not in grammar:
                    raise SyntaxError(f"索引到的ksf协议文件列表中找不到对应的依赖，[{curr}]存在错误的依赖[{inc_path}]")

                if not g_files.add_edge(inc_path.name, curr):
                    warnings.warn(f"[{curr}]重复添加依赖[{inc_path.name}]", category=SyntaxWarning, stacklevel=4,
                                  source=None)

                g_files_reverse.add_edge(curr, inc_path.name)

    # 2. 解析一下包含头，看看有没有循环包含
    def check_if_cycled():
        cycle = g_files.find_cycle()
        if cycle:
            raise Exception(f"解析的头文件依赖出现回环:{' -> '.join(str(node) for node in cycle)}, 请检查协议文件！")

    check_if_cycled()

    # 判断名称重复了
    flag_dicts = defaultdict(dict)

    def is_name_collision(flag, name, filename=None):
        if name not in flag_dicts[flag]:
            flag_dicts[flag][name] = filename
            return False
        else:
            return True

    def has_name(flag, name, filename=None):
        if name not in flag_dicts[flag]:
            return False, f"该名称[{name}]不存在"

        if filename is None or flag_dicts[flag][name] is None:
            return True, ""

        # 如果filename不为None, 则需要递归的判断一下该名称所在的文件是否位于当前文件所能依赖的所有文件中
        if flag_dicts[flag][name] in g_files_reverse.get_all_prev(filename, with_self=True):
            return True, ""
        else:
            return False, f"该名称[{name}]不位于被[{filename}]依赖的文件中"

    def traverse_all_file(func):
        # 处理单点文件
        for cur_file in grammar:
            if not g_files.has_node(cur_file):
                func(cur_file)

        # 按照依赖顺序遍历文件
        g_files.bfs_dag(func)

    # traverse_all_file(lambda a:print(a))
    # exit(0)
    def check_module_depends(filename):
        for module_desc in grammar[filename]['module']:
            # print(f'检测文件[{filename}]的模块[{module_desc["name"]}]中...')
            module_name = module_desc["name"]

            # 判断类型是否合法
            def is_valid(etype):
                # native 为原生类型，不用解析
                if etype['type'] == 'native':
                    return True

                # vector 需递归解析
                if etype['type'] == 'vector':
                    return is_valid(etype['value_type'])

                # map 则需要 key_type和value_type双解析
                if etype['type'] == 'map':
                    return is_valid(etype['value_type']) and is_valid(etype['key_type'])

                valid, err_msg = has_name('element',
                                          (module_name if etype['module'] is None else etype['module']) + '.' + etype[
                                              'name'], filename)
                if not valid:
                    raise SyntaxError(err_msg)

            for element in module_desc['elements']:
                if 'name' in element and is_name_collision('element', module_name + '.' + element['name'], filename):
                    raise SyntaxError(f"不允许有重复的模块元素命名{module_name + '.' + element['name']}")

                # pretty_print(element)
                # 判断是否成立的原则：
                # 1. 类(结构体or枚举)之前是否有其他类定义，包括在依赖的文件中定义的
                # 2. 接口中定义的类是否被定义
                if element['element'] == 'enum':
                    for member in element['member']:
                        if is_name_collision('enum_member', module_name + '.' + element['name'] + '.' + member['name']):
                            raise SyntaxError(f"枚举定义[{module_name + '.' + element['name']}]中有重复"
                                              f"[{module_name + '.' + element['name'] + '.' + member['name']}]")
                elif element['element'] == 'struct':
                    # 先确定自身的变量类型是否是已经存在的，不存在则退出
                    # pretty_print(element['variable'])
                    for var in element['variable']:
                        if is_name_collision('struct_variable',
                                             module_name + '.' + element['name'] + '.' + var['name']):
                            raise SyntaxError(
                                f"结构体[{module_name + '.' + element['name']}]不允许有重名的变量{var['name']}")

                        value_type = var['value_type']
                        is_valid(value_type)
                elif element['element'] == 'interface':
                    # 对于所有函数，判断 返回值类型、参数类型是否合法
                    # 对于所有函数，判断 参数名是否重复
                    for operator in element['operator']:
                        if is_name_collision('operator_name',
                                             module_name + '.' + element['name'] + '.' + operator['name']):
                            raise SyntaxError(
                                f"不允许有重名的接口函数{module_name + '.' + element['name'] + '.' + operator['name']}")

                        def has_return():
                            return operator['return_type']['name'] != 'void'

                        has_return() and is_valid(operator['return_type'])
                        for var in operator['variable']:
                            is_valid(var['value_type'])
                            if is_name_collision('operator_params',
                                                 module_name + '.' + element['name'] + '.' + operator['name'] + '.' +
                                                 var['name']):
                                raise SyntaxError(f"接口[{module_name + '.' + element['name']}]定义中有重复参数名"
                                                  f"[{module_name + '.' + element['name'] + '.' + operator['name'] + '.' + var['name']}]")
                elif element['element'] == 'key':
                    # 对于所有key值, 保证结构体类型合法，参数名存在于结构体定义中，且Key只能定义在对应的模块中，且每一个结构体只能定义一个key
                    valid, err_msg = has_name('element', module_name + '.' + element['struct'], filename)
                    if not valid:
                        raise SyntaxError(f"定义key时，结构体的{err_msg}")

                    if is_name_collision('key', module_name + '.' + element['struct'], filename):
                        raise SyntaxError(
                            f"一个结构体只能定义一个key，结构体[{module_name + '.' + element['struct']}]有重复的key定义")

                    for arg in element['args']:
                        valid, err_msg = has_name('struct_variable', module_name + '.' + element['struct'] + '.' + arg)
                        if not valid:
                            raise SyntaxError(f"定义key时，参数的{err_msg}")

                elif element['element'] == 'constant':
                    pass
                else:
                    raise SyntaxError(f"无法处理的元素[{element.__str__()}]")

    def parse_file_to_object(filename):
        # 因为已经经过检测了，所以全局只需要根据文件进行相应的组织
        for module_desc in grammar[filename]['module']:
            # print(f'解析文件[{filename}]的模块[{module_desc["name"]}]中...')
            module_name = module_desc["name"]

            Ksf.file2module[filename].append(module_name)
            Ksf.module2file[module_name].append(filename)

            for element in module_desc['elements']:
                if element['element'] == 'enum':
                    ksf_enum = KsfEnum(file=filename,
                                       module=module_name,
                                       name=element['name'],
                                       member=element['member'],
                                       comment=element['comment']
                                       )

                    Ksf.add_enum(ksf_enum)
                elif element['element'] == 'struct':
                    ksf_struct = KsfStruct(file=filename,
                                           module=module_name,
                                           name=element['name'],
                                           variable=element['variable'],
                                           comment=element['comment']
                                           )

                    Ksf.add_struct(ksf_struct)
                elif element['element'] == 'interface':
                    ksf_interface = KsfInterface(file=filename,
                                                 module=module_name,
                                                 name=element['name'],
                                                 operator=element['operator'],
                                                 comment=element['comment']
                                                 )

                    Ksf.add_interface(ksf_interface)
                elif element['element'] == 'key':
                    Ksf.add_struct_key(module=module_name,
                                       struct=element['struct'],
                                       arg=element['args']
                                       )
                elif element['element'] == 'constant':
                    ksf_const = KsfConst(file=filename,
                                         module=module_name,
                                         value_type=element['type'],
                                         name=element['name'],
                                         value=element['value'],
                                         comment=element['comment']
                                         )

                    Ksf.add_constant(ksf_const)
                else:
                    raise SyntaxError(f"无法处理的元素[{element.__str__()}]")

    # 检查模块依赖关系，确定合法
    traverse_all_file(check_module_depends)

    # 开始转换文件到结构体
    traverse_all_file(parse_file_to_object)


def find_file(file_path, search_dirs, with_curr_dir_first):
    file = Path(file_path)
    if with_curr_dir_first and file.exists():
        return file

    if file.is_absolute() and not file.exists():
        return None

    for dir in search_dirs:
        search_file = Path(dir) / file_path
        if search_file.exists():
            return search_file

    return None


def generate(files, search_dirs: set, with_curr_dir_first: bool):
    parser_grammar = {}

    def parse_depends(file):
        file = find_file(file, search_dirs, with_curr_dir_first)
        if file is None:
            raise FileNotFoundError(f"文件[{file}]没有找到")

        Ksf.add_file(file)

        # 创建语法分析器
        parser = KsfParser()
        if file not in parser_grammar:
            with file.open() as f:
                curr_file_grammar = parser.parse(f.read())
                parser_grammar[file.name] = curr_file_grammar

                if len(curr_file_grammar['include']) != 0:
                    for include in curr_file_grammar['include']:
                        # 在search路径中添加当前文件所在路径
                        curr_search_dirs = search_dirs
                        curr_search_dirs.add(str(file.parent.resolve()))

                        inc_path = find_file(include, curr_search_dirs, with_curr_dir_first)
                        if inc_path is None:
                            raise FileNotFoundError(f"依赖的头文件[{include}]未找到")

                        Ksf.add_include(file, inc_path)

                        if str(Path(inc_path).name) not in parser_grammar:
                            parse_depends(inc_path)

    for file in files:
        parse_depends(file)

    # print(parser_grammar.keys())

    generate_ksf_ast(parser_grammar, search_dirs)
    return Ksf


# 测试用例
if __name__ == '__main__':

    # 读取文件
    parser_grammar = {}
    file_path = [
        'example/enum_simple.ksf',
        'example/const_definition.ksf',
        'example/struct_simple.ksf'
    ]

    # (默认)生成单文件模式ast，保证所有数据都完全合法解析
    generate(file_path)

    for enum in Ksf.enums:
        print(Ksf.enums[enum].id)

    for struct in Ksf.structs:
        print(Ksf.structs[struct].id)
        print(Ksf.structs[struct].key_fields)

    for interface in Ksf.interfaces:
        print(Ksf.interfaces[interface].id)

    for const in Ksf.consts:
        print(Ksf.consts[const].id)
